@@ -276,8 +276,8 @@ void print_map(reiserfs_filsys_t * fs);
 struct saved_item {
     struct item_head si_ih;
     unsigned long si_block;
-    int si_item_num;
-    struct saved_item * si_next; /* list of items having the same key */
+    int si_item_num, si_entry_pos;
+    struct saved_item *si_next; /* list of items having the same key */
 };
 
 /*
