@@ -335,7 +335,7 @@ void warn_what_will_be_done (char * file_name, struct fsck_data * data)
 {
     FILE * warn_to;
 
-    warn_to = (data->progress ?: stderr);
+    warn_to = (data->progress ? data->progress : stderr);
 
     if (data->mode == FSCK_REBUILD)
 	reiserfs_warning (warn_to, REBUILD_WARNING, file_name);
@@ -413,7 +413,7 @@ void warn_what_will_be_done (char * file_name, struct fsck_data * data)
 
     reiserfs_warning (warn_to, "Will put log info to '%s'\n", 
 	(data->log == stdout) ? "stdout" : 
-				(data->log_file_name ? : "fsck.run"));
+	(data->log_file_name ? data->log_file_name : "fsck.run"));
     
     if (!(data->options & OPT_YES) && !user_confirmed (warn_to, "\nDo you want to "
 	"run this program?[N/Yes] (note need to type Yes if you do):", "Yes\n"))
