@@ -229,15 +229,15 @@ static void callback_new_badblocks(reiserfs_filsys_t *fs,
 	tmp_ih = get_ih(badblock_path);
 	ind_item = (__u32 *)get_item(badblock_path);
 
-	for (i = 0; i < I_UNFM_NUM(tmp_ih); i++, ind_item++) {
+	for (i = 0; i < I_UNFM_NUM(tmp_ih); i++) {
 		if (reiserfs_bitmap_test_bit(fs->fs_badblocks_bm, 
-					     le32_to_cpu(*ind_item)))
+					     d32_get (ind_item, i)))
 		{
 			message("Block %u is marked as bad already.", 
-				le32_to_cpu(*ind_item));
+				d32_get (ind_item, i));
 			
 			reiserfs_bitmap_clear_bit(fs->fs_badblocks_bm, 
-						  le32_to_cpu(*ind_item));
+						  d32_get (ind_item, i));
 		}
 	}
 	
@@ -254,9 +254,9 @@ static void callback_clear_badblocks(reiserfs_filsys_t *fs,
 	tmp_ih = get_ih(badblock_path);
 	ind_item = (__u32 *)get_item(badblock_path);
 
-	for (i = 0; i < I_UNFM_NUM(tmp_ih); i++, ind_item++) {
+	for (i = 0; i < I_UNFM_NUM(tmp_ih); i++) {
 		reiserfs_bitmap_clear_bit(fs->fs_bitmap2, 
-					  le32_to_cpu(*ind_item));
+					  d32_get(ind_item, i));
 	}
 	
 	pathrelse (badblock_path);
